@@ -1,7 +1,7 @@
 /* sha512.c - Functions to compute SHA512 and SHA384 message digest of files or
    memory blocks according to the NIST specification FIPS-180-2.
 
-   Copyright (C) 2005-2006, 2008-2016 Free Software Foundation, Inc.
+   Copyright (C) 2005-2006, 2008-2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by David Madore, considerably copypasting from
    Scott G. Miller's sha1.c.  Modifications for partial file
@@ -33,18 +33,11 @@
 # include "unlocked-io.h"
 #endif
 
+#include <byteswap.h>
 #ifdef WORDS_BIGENDIAN
 # define SWAP(n) (n)
 #else
-# define SWAP(n) \
-    u64or (u64or (u64or (u64shl (n, 56),                                \
-                         u64shl (u64and (n, u64lo (0x0000ff00)), 40)),  \
-                  u64or (u64shl (u64and (n, u64lo (0x00ff0000)), 24),   \
-                         u64shl (u64and (n, u64lo (0xff000000)),  8))), \
-           u64or (u64or (u64and (u64shr (n,  8), u64lo (0xff000000)),   \
-                         u64and (u64shr (n, 24), u64lo (0x00ff0000))),  \
-                  u64or (u64and (u64shr (n, 40), u64lo (0x0000ff00)),   \
-                         u64shr (n, 56))))
+# define SWAP(n) bswap_64 (n)
 #endif
 
 #define BLOCKSIZE 32768
@@ -172,22 +165,24 @@ sha384_finish_ctx (struct sha512_ctx *ctx, void *resbuf)
   return sha384_read_ctx (ctx, resbuf);
 }
 
-/* Compute SHA512 message digest for bytes read from STREAM.  The
-   resulting message digest number will be written into the 64 bytes
-   beginning at RESBLOCK.  */
-int
-sha512_stream (FILE *stream, void *resblock, off_t readbytes)
+/* Compute message digest for bytes read from STREAM using algorithm ALG.
+   Write the message digest into RESBLOCK, which contains HASHLEN bytes.
+   The initial and finishing operations are INIT_CTX and FINISH_CTX.
+   Return zero if and only if successful.  */
+static int
+shaxxx_stream (FILE *stream, char const *alg, void *resblock, off_t readbytes,
+               ssize_t hashlen, void (*init_ctx) (struct sha512_ctx *),
+               void *(*finish_ctx) (struct sha512_ctx *, void *))
 {
-  struct sha512_ctx ctx;
-  size_t sum, target;
-  off_t total = (off_t) 0;
-
   char *buffer = malloc (BLOCKSIZE + 72);
   if (!buffer)
     return 1;
 
   /* Initialize the computation context.  */
-  sha512_init_ctx (&ctx);
+  struct sha512_ctx ctx;
+  init_ctx (&ctx);
+  size_t sum, target;
+  off_t total = (off_t) 0;
 
   /* Iterate over full file contents.  */
   while (1)
@@ -205,6 +200,14 @@ sha512_stream (FILE *stream, void *resblock, off_t readbytes)
       /* Read block.  Take care for partial reads.  */
       while (1)
         {
+          /* Either process a partial fread() from this loop,
+             or the fread() in afalg_stream may have gotten EOF.
+             We need to avoid a subsequent fread() as EOF may
+             not be sticky.  For details of such systems, see:
+             https://sourceware.org/bugzilla/show_bug.cgi?id=1190  */
+          if (feof (stream))
+            goto process_partial_block;
+
           n = fread (buffer + sum, 1, target - sum, stream);
           sum += n;
 
@@ -223,12 +226,6 @@ sha512_stream (FILE *stream, void *resblock, off_t readbytes)
                 }
               goto process_partial_block;
             }
-
-          /* We've read at least one byte, so ignore errors.  But always
-             check for EOF, since feof may be true even though N > 0.
-             Otherwise, we could end up calling fread after EOF.  */
-          if (feof (stream))
-            goto process_partial_block;
         }
       total += (off_t) sum;
 
@@ -249,89 +246,25 @@ sha512_stream (FILE *stream, void *resblock, off_t readbytes)
     sha512_process_bytes (buffer, sum, &ctx);
 
   /* Construct result in desired memory.  */
-  sha512_finish_ctx (&ctx, resblock);
+  finish_ctx (&ctx, resblock);
   free (buffer);
   return 0;
 }
 
-/* FIXME: Avoid code duplication */
+int
+sha512_stream (FILE *stream, void *resblock, off_t readbytes)
+{
+  return shaxxx_stream (stream, "sha512", resblock, readbytes,
+                        SHA512_DIGEST_SIZE, sha512_init_ctx,
+                        sha512_finish_ctx);
+}
+
 int
 sha384_stream (FILE *stream, void *resblock, off_t readbytes)
 {
-  struct sha512_ctx ctx;
-  size_t sum, target;
-  off_t total = (off_t) 0;
-
-  char *buffer = malloc (BLOCKSIZE + 72);
-  if (!buffer)
-    return 1;
-
-  /* Initialize the computation context.  */
-  sha384_init_ctx (&ctx);
-
-  /* Iterate over full file contents.  */
-  while (1)
-    {
-      /* We read the file in blocks of BLOCKSIZE bytes.  One call of the
-         computation function processes the whole buffer so that with the
-         next round of the loop another block can be read.  */
-      size_t n;
-      if ((readbytes < (off_t) 1) || ((readbytes - total) > (off_t) BLOCKSIZE))
-        target = BLOCKSIZE;
-      else
-        target = (size_t) (readbytes - total);
-      sum = 0;
-
-      /* Read block.  Take care for partial reads.  */
-      while (1)
-        {
-          n = fread (buffer + sum, 1, target - sum, stream);
-          sum += n;
-
-          if (sum == BLOCKSIZE)
-            break;
-
-          if (n == 0)
-            {
-              /* Check for the error flag IFF N == 0, so that we don't
-                 exit the loop after a partial read due to e.g., EAGAIN
-                 or EWOULDBLOCK.  */
-              if (ferror (stream))
-                {
-                  free (buffer);
-                  return 1;
-                }
-              goto process_partial_block;
-            }
-
-          /* We've read at least one byte, so ignore errors.  But always
-             check for EOF, since feof may be true even though N > 0.
-             Otherwise, we could end up calling fread after EOF.  */
-          if (feof (stream))
-            goto process_partial_block;
-        }
-      total += (off_t) sum;
-
-      /* If end of file is reached, end the loop.  */
-      if ((n == 0) || (sum == 0) || (total == readbytes))
-        goto process_partial_block;
-
-      /* Process buffer with BLOCKSIZE bytes.  Note that
-                        BLOCKSIZE % 128 == 0
-       */
-      sha512_process_block (buffer, BLOCKSIZE, &ctx);
-    }
-
- process_partial_block:;
-
-  /* Process any remaining bytes.  */
-  if (sum > 0)
-    sha512_process_bytes (buffer, sum, &ctx);
-
-  /* Construct result in desired memory.  */
-  sha384_finish_ctx (&ctx, resblock);
-  free (buffer);
-  return 0;
+  return shaxxx_stream (stream, "sha384", resblock, readbytes,
+                        SHA384_DIGEST_SIZE, sha384_init_ctx,
+                        sha384_finish_ctx);
 }
 
 /* Compute SHA512 message digest for LEN bytes beginning at BUFFER.  The
@@ -386,7 +319,8 @@ sha512_process_bytes (const void *buffer, size_t len, struct sha512_ctx *ctx)
           sha512_process_block (ctx->buffer, ctx->buflen & ~127, ctx);
 
           ctx->buflen &= 127;
-          /* The regions in the following copy operation cannot overlap.  */
+          /* The regions in the following copy operation cannot overlap,
+             because ctx->buflen < 128 ≤ (left_over + add) & ~127.  */
           memcpy (ctx->buffer,
                   &((char *) ctx->buffer)[(left_over + add) & ~127],
                   ctx->buflen);
@@ -399,7 +333,7 @@ sha512_process_bytes (const void *buffer, size_t len, struct sha512_ctx *ctx)
   /* Process available complete blocks.  */
   if (len >= 128)
     {
-#if !_STRING_ARCH_unaligned
+#if !(_STRING_ARCH_unaligned || _STRING_INLINE_unaligned)
 # define UNALIGNED_P(p) ((uintptr_t) (p) % alignof (u64) != 0)
       if (UNALIGNED_P (buffer))
         while (len > 128)
@@ -428,6 +362,8 @@ sha512_process_bytes (const void *buffer, size_t len, struct sha512_ctx *ctx)
         {
           sha512_process_block (ctx->buffer, 128, ctx);
           left_over -= 128;
+          /* The regions in the following copy operation cannot overlap,
+             because left_over ≤ 128.  */
           memcpy (ctx->buffer, &ctx->buffer[16], left_over);
         }
       ctx->buflen = left_over;
